@@ -214,6 +214,7 @@ class DearBagPlayer:
     def update(self):
         self.timelineUpdate()
         self.curPointUpdate()
+        self.checkLastPlotTab()
 
     def timelineUpdate(self):
         # Update head, index, and rendering
@@ -270,6 +271,14 @@ class DearBagPlayer:
         # Support both list or np.array
         return bisect.bisect(time_series, timestamp) - 1
 
+    def checkLastPlotTab(self):
+        """
+        Check if last plot tab is closed. If yes, create a new one.
+        """
+        self.deleteClosedTab()
+        if len(dpg.get_item_children(self.tab_bar)[1]) == 1:
+            self.addPlotPageCb("Add Plot Button", None, None)
+
     # -----------------------------------------
     # Plots
     # -----------------------------------------
@@ -342,8 +351,8 @@ class DearBagPlayer:
             dpg.add_line_series(datax[1], datay[1], label=label, parent=yaxis)
 
             # Add button to legend right click bar
-            dpg.add_button(label="Delete Series", user_data=dpg.last_item(), parent=dpg.last_item(),
-                           callback=lambda s, a, u: dpg.delete_item(u))
+            self.addLegendClickedMenu(dpg.last_item())
+
             old_user_data = dpg.get_item_user_data(yaxis)
             if old_user_data is None:
                 old_user_data = list()
@@ -366,8 +375,8 @@ class DearBagPlayer:
             dpg.add_line_series(datax[1], datay[1], label=label, parent=yaxis)
 
             # Add button to legend right click bar
-            dpg.add_button(label="Delete Series", user_data=dpg.last_item(), parent=dpg.last_item(),
-                           callback=lambda s, a, u: dpg.delete_item(u))
+            self.addLegendClickedMenu(dpg.last_item())
+
             old_user_data = dpg.get_item_user_data(yaxis)
             if old_user_data is None:
                 old_user_data = list()
@@ -384,13 +393,28 @@ class DearBagPlayer:
                 label = bag_name + ":" + data[2]
                 dpg.add_line_series(data[0], data[1], label=label, parent=yaxis)
                 # Add button to legend right click bar
-                dpg.add_button(label="Delete Series", user_data=dpg.last_item(), parent=dpg.last_item(),
-                               callback=lambda s, a, u: dpg.delete_item(u))
+                self.addLegendClickedMenu(dpg.last_item())
 
         # Clean drop data & fit plot regions
         self._resetValue()
         self._fitAxesData(dpg.get_item_info(yaxis)["parent"])
 
+    def addLegendClickedMenu(self, series_tag):
+        # Add button to legend right click bar
+        dpg.add_button(label="Delete Selected Series", user_data=series_tag, parent=series_tag,
+                       callback=self.deleteSelectedSeriesCb)
+        dpg.add_button(label="Delete All Series", user_data=series_tag, parent=series_tag,
+                       callback=self.deleteAllSeriesCb)
+
+    def deleteSelectedSeriesCb(self, sender, app_data, user_data):
+        """user_data stores the tag of selected series"""
+        dpg.delete_item(user_data)
+
+    def deleteAllSeriesCb(self, sender, app_data, user_data):
+        """user_data stores the tag of selected series"""
+        plot = dpg.get_item_parent(user_data)
+        dpg.delete_item(plot, children_only=True)
+
     def plotDropCallback(self, sender, app_data, user_data):
         yaxis = dpg.get_item_info(sender)["children"][1][1]
         self.commonDropCallback(yaxis, app_data)
@@ -398,6 +422,24 @@ class DearBagPlayer:
     def axisDropCallback(self, sender, app_data, user_data):
         self.commonDropCallback(sender, app_data)
 
+    def dragTopicPayloadCb(self, sender, app_data, user_data):
+        """
+        :param sender: dragged selectable item (topic)
+        :param app_data: list of selected items (topics) in data pool
+        :param user_data: None
+        """
+        # Append item if not selected
+        if sender not in app_data:
+            app_data.append(sender)
+
+        # Update payload text
+        payload = dpg.get_item_children(sender, slot=3)[0]
+        payload_text = dpg.get_item_children(payload)[1][0]
+        dpg.configure_item(
+            payload_text,
+            default_value=f"{len(app_data)} series to plot"
+        )
+
     def addPlot(self, title="", x_label="", y_label="", height=200, width=300,
                 equal_aspects=False, drop_plot_enabled=True):
         
@@ -408,7 +450,7 @@ class DearBagPlayer:
             plot_drop_callback = None
             axis_drop_callback = None
 
-        with dpg.plot(label=title, height=height, width=width, equal_aspects=equal_aspects, 
+        with dpg.plot(label=title, height=height, width=width, equal_aspects=equal_aspects,
                       payload_type="plotting", drop_callback=plot_drop_callback):
             dpg.add_plot_legend()
             dpg.add_plot_axis(dpg.mvXAxis, label=x_label)
@@ -445,10 +487,43 @@ class DearBagPlayer:
 
     def addPlotPageCb(self, sender, app_data, user_data):
         dpg.get_item_user_data(self.tab_bar)['plot_pages'] += 1
-        with dpg.tab(label=f"Plot {dpg.get_item_user_data(self.tab_bar)['plot_pages']}", parent=self.tab_bar,
-                     before="Add Plot Button"):
+        with dpg.tab(label=f"Plot {dpg.get_item_user_data(self.tab_bar)['plot_pages']}",
+                     parent=self.tab_bar, closable=True) as tab_tag:
+            dpg.bind_item_handler_registry(tab_tag, "tab_clicked_handler")
             self.createSubplots()
 
+    def tabClickedMenuCb(self, sender, app_data, user_data):
+        """
+        :param sender: tab_bar
+        :param app_data: [clicked_mouse_button (0-left, 1-right, 2-middle), clicked_tab]
+        :param user_data: None
+        """
+        pos = dpg.get_mouse_pos(local=False)
+        if app_data[0] == 1:  # right-clicked
+            with dpg.window(pos=pos, min_size=[70, 15], popup=True, autosize=False):
+                dpg.add_button(label="Rename", user_data=(app_data[1], pos), callback=self.renamePlotTabCb)
+
+    def renamePlotTabCb(self, sender, app_data, user_data):
+        """
+        :param sender: Rename button
+        :param app_data: None
+        :param user_data: [clicked_tab, clicked_pos (x, y)]
+        """
+        pos = user_data[1]
+        with dpg.window(pos=pos, min_size=[120, 15], no_title_bar=True, no_scrollbar=True) as rename_win:
+            dpg.add_input_text(label="", hint="<new name>", on_enter=True,
+                               user_data=(user_data[0], rename_win),
+                               callback=self.renameWindowCb)
+
+    def renameWindowCb(self, sender, app_data, user_data):
+        """
+        :param sender: input_text item
+        :param app_data: New name of the tab
+        :param user_data: (tab_to_rename, rename_win)
+        """
+        dpg.configure_item(user_data[0], label=app_data)
+        dpg.delete_item(user_data[1])  # delete rename window
+
     def splitHorizontallyCb(self, sender, app_data, user_data):
         subplots = dpg.get_item_user_data(self.tab_bar)['act_plot']
         cols = dpg.get_item_configuration(subplots)['cols']
@@ -480,11 +555,28 @@ class DearBagPlayer:
             dpg.delete_item(yaxis, children_only=True)
 
     def updateActCb(self, sender, app_data, user_data):
-        # app_data is the activated tab
+        """
+        Triggered when activated tab is changed.
+
+        Actions that won't trigger this callback function:
+        - Create a new tab by clicking the '+' button
+        - Delete a tab that is not activated
+        - Delete the last tab
+
+        :param sender: tag of the tab_bar
+        :param app_data: the activated tab
+        :param user_data: {"act_tab": tag, "act_plot": tag, "plot_pages": int}
+        """
+        self.deleteClosedTab()
         self.clearTimeLinesAndPoints()
         dpg.get_item_user_data(sender)['act_tab'] = app_data
         dpg.get_item_user_data(sender)['act_plot'] = dpg.get_item_children(app_data)[1][0]
 
+    def deleteClosedTab(self):
+        for tab in dpg.get_item_children(self.tab_bar)[1]:
+            if not dpg.get_item_configuration(tab)['show']:
+                dpg.delete_item(tab)
+
     # -----------------------------------------
     # File Import
     # -----------------------------------------
@@ -519,6 +611,7 @@ class DearBagPlayer:
                     items.append(
                         dpg.add_selectable(
                             label=entity, payload_type="plotting", callback=_update_count,
+                            drag_callback=self.dragTopicPayloadCb,
                             user_data=(
                                 database[topic]["timestamp"],
                                 database[topic][entity],
@@ -527,12 +620,10 @@ class DearBagPlayer:
                         )
                     )
 
-                    # TODO: drag to plot directly when no item is selected,
-                    #  cancel selected items when click other windows
                     with dpg.drag_payload(parent=dpg.last_item(),
-                                          drag_data=(dpg.get_item_user_data(self.data_pool_window)),
+                                          drag_data=dpg.get_item_user_data(self.data_pool_window),
                                           payload_type="plotting"):
-                        dpg.add_text(f"{len(dpg.get_item_user_data(self.data_pool_window))} series to plot")
+                        dpg.add_text("drag series to plot")
 
     # -----------------------------------------
     # Main Entry
@@ -597,24 +688,7 @@ class DearBagPlayer:
         # Data series list
         self.data_pool_window = dpg.add_window(label="Data Pool", height=650, width=400, user_data=list())
 
-        with dpg.window(label="Plot Window", pos=(420, 0), height=800, width=810):
-            with dpg.group(horizontal=True):
-                dpg.add_button(label="Split Horizontally", callback=self.splitHorizontallyCb)
-                dpg.add_button(label="Split Vertically", callback=self.splitVerticallyCb)
-                dpg.add_button(label="Clear", callback=self.clearCb)
-
-            with dpg.tab_bar(user_data={"act_tab": None, "act_plot": None, "plot_pages": 1},
-                             callback=self.updateActCb) as self.tab_bar:
-                with dpg.tab(label=f"Plot {dpg.get_item_user_data(self.tab_bar)['plot_pages']}"):
-                    dpg.get_item_user_data(self.tab_bar)['act_tab'] = dpg.last_item()
-                    with dpg.subplots(rows=1, columns=1, no_title=True, height=600, width=800):
-                        dpg.get_item_user_data(self.tab_bar)['act_plot'] = dpg.last_item()
-                        self.addPlot()
-                dpg.add_tab_button(label="+", tag="Add Plot Button", callback=self.addPlotPageCb)
-
-            self.__timeline.createWidgets()
-
-        # handlers
+        # Handlers
         with dpg.handler_registry(tag="__demo_keyboard_handler"):  # show=True by default
             dpg.add_key_down_handler(key=dpg.mvKey_Control)
             dpg.add_key_release_handler(key=dpg.mvKey_Control)
@@ -626,6 +700,29 @@ class DearBagPlayer:
         for handler in dpg.get_item_children("__demo_keyboard_handler", 1):
             dpg.set_item_callback(handler, self.eventHandler)
 
+        with dpg.item_handler_registry(tag="tab_clicked_handler"):
+            dpg.add_item_clicked_handler(callback=self.tabClickedMenuCb)
+
+        # Plot window
+        with dpg.window(label="Plot Window", pos=(420, 0), height=800, width=810):
+            with dpg.group(horizontal=True):
+                dpg.add_button(label="Split Horizontally", callback=self.splitHorizontallyCb)
+                dpg.add_button(label="Split Vertically", callback=self.splitVerticallyCb)
+                dpg.add_button(label="Clear", callback=self.clearCb)
+
+            with dpg.tab_bar(user_data={"act_tab": None, "act_plot": None, "plot_pages": 1},
+                             reorderable=True, callback=self.updateActCb) as self.tab_bar:
+                with dpg.tab(label=f"Plot {dpg.get_item_user_data(self.tab_bar)['plot_pages']}",
+                             closable=True) as tab_tag:
+                    dpg.bind_item_handler_registry(tab_tag, "tab_clicked_handler")
+                    dpg.get_item_user_data(self.tab_bar)['act_tab'] = tab_tag
+                    with dpg.subplots(rows=1, columns=1, no_title=True, height=600, width=800):
+                        dpg.get_item_user_data(self.tab_bar)['act_plot'] = dpg.last_item()
+                        self.addPlot()
+                dpg.add_tab_button(label="+", tag="Add Plot Button", callback=self.addPlotPageCb, trailing=True)
+
+            self.__timeline.createWidgets()
+
         # The Primary Window
         # dpg.set_primary_window("Primary Window", True)
 
